package com.gree.mall.manager.utils.oss;

import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.model.GeneratePresignedUrlRequest;
import com.aliyun.oss.model.PutObjectRequest;
import com.aliyun.oss.model.PutObjectResult;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.InputStream;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;

@Service
@Slf4j
public class OSSUtil {

    @Value("${ali.access.key.id}")
    private String ACCESS_KEY_ID;
    @Value("${ali.access.key.secert}")
    private String ACCESS_KEY_SECERT;

    @Value("${ali.oss.bucket.name}")
    private String OSS_BUCKET_NAME;
    @Value("${ali.oss.endpoint.ww}")
    private String OSS_ENDPOINT_WW;
    @Value("${ali.oss.endpoint}")
    private String OSS_ENDPOINT;
    @Value("${spring.profiles.active}")
    private String active;
    @Value("${sys.url}")
    private String sysUrl;
    @Value("${server.servlet.context-path}")
    private String servletPath;

    public String getAccessUrl(){
        return "https://" + OSS_BUCKET_NAME +"."+ OSS_ENDPOINT_WW;
    }

    public String getAccessUrl2(){
        return sysUrl + servletPath + "/img/get?key=";
    }

    public String getFilePath() {
        String month = new SimpleDateFormat("yyyy-MM").format(new Date());
        return month;
    }

    public OSS buildClient(){
        return new OSSClientBuilder().build(!active.equals("prd") ? OSS_ENDPOINT_WW : OSS_ENDPOINT, ACCESS_KEY_ID, ACCESS_KEY_SECERT);
    }

    /**
     * 上传到oss
     *
     * @param ossObjectKey 附件的唯一路径名称
     * @param file         附件
     * @return
     */
    public boolean uploadFile(String ossObjectKey, File file) {
        PutObjectResult putObjectResult = buildClient().putObject(new PutObjectRequest(OSS_BUCKET_NAME, ossObjectKey, file));
        log.info("putObjectResult" + putObjectResult);
        return true;
    }

    public boolean uploadFile(String ossObjectKey, byte[] file) {
        PutObjectResult putObjectResult = buildClient().putObject(new PutObjectRequest(OSS_BUCKET_NAME, ossObjectKey, new ByteArrayInputStream(file)));
        log.info("putObjectResult" + putObjectResult);
        return true;
    }

    public boolean uploadFile(String ossObjectKey, ByteArrayInputStream byteArrayInputStream) {
        PutObjectResult putObjectResult = buildClient().putObject(new PutObjectRequest(OSS_BUCKET_NAME, ossObjectKey, byteArrayInputStream));
        log.info("putObjectResult" + putObjectResult);
        return true;
    }

    public boolean uploadFile(String ossObjectKey, InputStream inputStream) {
        PutObjectResult putObjectResult = buildClient().putObject(new PutObjectRequest(OSS_BUCKET_NAME, ossObjectKey, inputStream));
        log.info("putObjectResult" + putObjectResult);
        return true;
    }

    public String getUrlWw(String key) {
        return getUrlWw(key, null);
    }

    public String getUrlWw(String key, MediaType mediaType) {
        if (StringUtils.isEmpty(key)) {
            return "";
        }
        // 设置URL过期时间为1小时
        Date expiration = new Date(new Date().getTime() + 3600 * 1000);
        GeneratePresignedUrlRequest request = new GeneratePresignedUrlRequest(OSS_BUCKET_NAME, key);
        request.setExpiration(expiration);
        if (mediaType != null) {
            request.setContentType(mediaType.toString());
        }

        URL url = new OSSClientBuilder()
                .build("https://" + OSS_ENDPOINT_WW, ACCESS_KEY_ID, ACCESS_KEY_SECERT)
                .generatePresignedUrl(request);

        String resultUrl = url.toString();

        return resultUrl;
    }


}
